﻿using CarServiceContracts.BindingModels;
using CarServiceContracts.BusinessLogicsContracts;
using CarServiceContracts.SearchModels;
using CarServiceContracts.StorageContracts;
using CarServiceContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace CarServiceBusinessLogic.BusinessLogics
{
	public class WorkLogic : IWorkLogic
	{
		private readonly ILogger _logger;
		private readonly IWorkStorage _workStorage;
		public WorkLogic(ILogger<WorkLogic> logger, IWorkStorage workStorage)
		{
			_logger = logger;
			_workStorage = workStorage;
		}
		public List<WorkViewModel>? ReadList(WorkSearchModel? model)
		{
			_logger.LogInformation("ReadList. Id: {Id}", model?.Id);
			var list = model == null ? _workStorage.GetFullList() : _workStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count: {Count}", list.Count);
			return list;
		}
		public WorkViewModel? ReadElement(WorkSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. Id: {Id}", model.Id);
			var element = _workStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement found. Id: {Id}", element.Id);
			return element;
		}
		public bool Create(WorkBindingModel model)
		{
			CheckModel(model);
			if (_workStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}
		public bool Update(WorkBindingModel model)
		{
			CheckModel(model, false);
			if (_workStorage.Update(model) == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			return true;
		}
		public bool Delete(WorkBindingModel model)
		{
			CheckModel(model, false);
			_logger.LogInformation("Delete. Id: {Id}", model.Id);
			if (_workStorage.Delete(model) == null)
			{
				_logger.LogWarning("Delete operation failed");
				return false;
			}
			return true;
		}
		private void CheckModel(WorkBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			//Заполнено ли название?
			if (string.IsNullOrEmpty(model.Name))
			{
				_logger.LogWarning("Work name is empty");
				throw new ArgumentException("Не введено название");
			}
			//Название уникально?
			var existingWork = _workStorage.GetElement(new() { Name = model.Name });
			if (existingWork != null)
			{
				_logger.LogWarning("Work name is not unique");
				throw new ArgumentException("Работа с таким названием уже есть");
			}
			//Название не больше 60 символов?
			if (model.Name.Length > 60)
			{
				_logger.LogWarning("Work name's length > 60");
				throw new ArgumentException("Не введено название");
			}
			//Цена больше 0?
			if (model.Price <= 0)
			{
				_logger.LogWarning("Work Price is <= 0");
				throw new ArgumentException("Цена должна быть больше 0");
			}
			//Длительность больше 0?
			if (model.Duration <= 0)
			{
				_logger.LogWarning("Work Duration is <= 0");
				throw new ArgumentException("Длительность должна быть больше 0");
			}
			_logger.LogInformation("Work. Id: {Id}", model.Id);
		}
	}
}
